function PrimeUI:BuffBar(data, parent)
    local x = self:ContentAlignement(data["Alignment"], data["Width"])
    local y = self.currentHeight
    if self.groupMode then
        x = data.X
        y = data.Y
    end

    local buffbar = ClassSpecificTools.DynamicSkillBar(parent, x, y, data["Width"], (data["BorderSize"] and data["BorderSize"] * 2 or 0) + 32 * data["Scale"], (data["BorderSize"] and data["BorderSize"] * 2 or 0) + 32 * data["Scale"], self:AlignmentToNumber(data["Alignment"]), true, true, data["Offset"], true)
    
    buffbar.SkillID = {}
    buffbar.UniqueNames = {}

    if self.groupMode then
        buffbar.moveBox = MoveBox(buffbar, data["Width"], data["Width"], data["NickName"] or "Buffbar")
        buffbar.savePosition = function()
            data["X"], data["Y"] = buffbar:GetPosition()
        end
    end

    if tablelength(data["Effects"]) > 0 or data["AllEffects"] then
        buffbar.effectFunction = function(effect)
            local name = effect:GetName()
            if data["Effects"][name] or data["AllEffects"] then
                if (not(data["OnlyBuffs"]) and not(data["OnlyDebuffs"])) or (data["OnlyDebuffs"] and effect:IsDebuff()) or (data["OnlyBuffs"] and not(effect:IsDebuff())) then
                    local highlight = data["AllHighlight"]
                    local icon = 0
                    local value = data["Effects"][name]
                    local time = value["Time"]
                    if value then
                        highlight = highlight or value["Highlight"]
                        if value["Icon"] == true then 
                            icon = effect:GetIcon()
                        else
                            icon = value["Icon"] 
                        end

                        if value["Fit_Icon"] and value["Icon"] and value["Icon"] ~= true then
                            if effect:GetIcon() ~= tonumber(value["Icon"]) then 
                                return 
                            end
                        end

                        if time == true then
                            time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                        end
                    else
                        icon = effect:GetIcon()
                    end

                    local ID = effect:GetID()
                    buffbar.SkillID[ID] = true
                    buffbar:AddLabel(ID, ClassSpecificTools.ClassTimerLabel(buffbar, 0, 0, 32, 32, icon, false, false, nil, 1, 1, false, false, true, data["Scale"], name, true, self.alignments[data["TextAlignment"]], self.alignments[data["StackTextAlignment"]], highlight, self:FontToFont(data["Font_Timer"]), self:FontToFont(data["Font_Stack"]), self:ColourTupleToTurbineColour(data["Font_Timer_Colour"]), self:ColourTupleToTurbineColour(data["Font_Stack_Colour"]), data["BorderSize"], self:ColourTupleToTurbineColour(data["BorderColour"]), self:ColourTupleToTurbineColour(data["HighlightColour"]), data["HighlightSize"]), time)
                    buffbar:ToggleLabel(ID, true, time)
                end
            end
        end

        for i = 1, self.player:GetEffects():GetCount() do
            buffbar.effectFunction(self.player:GetEffects():Get(i))
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", buffbar.effectFunction))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local ID = effect:GetID()
            if buffbar.SkillID[ID] then
                buffbar:RemoveLabel(ID)
            end

        end))
    end

    if tablelength(data["Skills"]) > 0 then
        self.skills = true
        for key, value in pairs(data["Skills"]) do
            if SkillEventHandler:SkillExists(key) then
                -- Add the callback for this skill slot
                if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill) 
                    local icon = value["Icon"]
                    if value["Icon"] == true then
                        icon = skill:GetSkillInfo():GetIconImageID()
                    end

                    local time = value["Time"]
                    if value["Time"] == true then
                        time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                    end

                    local highlight = data["AllHighlight"] or value["Highlight"]

                    if not(buffbar:LabelExists(key)) then
                        buffbar:AddLabel(key, ClassSpecificTools.ClassTimerLabel(buffbar, 0, 0, 32, 32, icon, false, false, nil, 1, 1, false, false, true, data["Scale"], key, true, self.alignments[data["TextAlignment"]], self.alignments[data["StackTextAlignment"]], highlight, self:FontToFont(data["Font_Timer"]), self:FontToFont(data["Font_Stack"]), self:ColourTupleToTurbineColour(data["Font_Timer_Colour"]), self:ColourTupleToTurbineColour(data["Font_Stack_Colour"]), data["BorderSize"], self:ColourTupleToTurbineColour(data["BorderColour"]), self:ColourTupleToTurbineColour(data["HighlightColour"]), data["HighlightSize"]), time)
                    end
                    buffbar:ToggleLabel(key, true, time)
                end))
            end
        end
    end

    if tablelength(data["PlayerHits"]) > 0 then
        for key, value in pairs(data["PlayerHits"]) do
            table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerCombat", function(args)
                local value = data["PlayerHits"][args.SkillName]
                if value then  
                    local highlight = data["AllHighlight"] or value["Highlight"]
                    if not(buffbar:LabelExists(key)) then
                        buffbar:AddLabel(key, ClassSpecificTools.ClassTimerLabel(buffbar, 0, 0, 32, 32, value["Icon"], false, false, nil, 1, 1, false, false, true, data["Scale"], key, true, self.alignments[data["TextAlignment"]], self.alignments[data["StackTextAlignment"]], highlight, self:FontToFont(data["Font_Timer"]), self:FontToFont(data["Font_Stack"]), self:ColourTupleToTurbineColour(data["Font_Timer_Colour"]), self:ColourTupleToTurbineColour(data["Font_Stack_Colour"]), data["BorderSize"], self:ColourTupleToTurbineColour(data["BorderColour"]), self:ColourTupleToTurbineColour(data["HighlightColour"]), data["HighlightSize"]), value["Time"])
                    end
                    buffbar:ToggleLabel(key, true, value["Time"])
                end
            end))
        end
    end

    if tablelength(data["PlayerHeal"]) > 0 then
        for key, value in pairs(data["PlayerHeal"]) do
            table.insert(self.playerHealCallbacks, AddCallback(ChatEventHandler, "PlayerHeal", function(args)
                local value = data["PlayerHeal"][args.SkillName]
                if value then  
                    local highlight = data["AllHighlight"] or value["Highlight"]
                    if not(buffbar:LabelExists(key)) then
                        buffbar:AddLabel(key, ClassSpecificTools.ClassTimerLabel(buffbar, 0, 0, 32, 32, value["Icon"], false, false, nil, 1, 1, false, false, true, data["Scale"], key, true, self.alignments[data["TextAlignment"]], self.alignments[data["StackTextAlignment"]], highlight, self:FontToFont(data["Font_Timer"]), self:FontToFont(data["Font_Stack"]), self:ColourTupleToTurbineColour(data["Font_Timer_Colour"]), self:ColourTupleToTurbineColour(data["Font_Stack_Colour"]), data["BorderSize"], self:ColourTupleToTurbineColour(data["BorderColour"]), self:ColourTupleToTurbineColour(data["HighlightColour"]), data["HighlightSize"]), value["Time"])
                    end
                    buffbar:ToggleLabel(key, true, value["Time"])
                end
            end))
        end
    end

    if tablelength(data["PlayerBenefit"]) > 0 then
        for key, value in pairs(data["PlayerBenefit"]) do
            table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "PlayerBenefit", function(args)
                local value = data["PlayerBenefit"][args.SkillName]
                if value then  
                    local highlight = data["AllHighlight"] or value["Highlight"]
                    if not(buffbar:LabelExists(key)) then
                        buffbar:AddLabel(key, ClassSpecificTools.ClassTimerLabel(buffbar, 0, 0, 32, 32, value["Icon"], false, false, nil, 1, 1, false, false, true, data["Scale"], key, true, self.alignments[data["TextAlignment"]], self.alignments[data["StackTextAlignment"]], highlight, self:FontToFont(data["Font_Timer"]), self:FontToFont(data["Font_Stack"]), self:ColourTupleToTurbineColour(data["Font_Timer_Colour"]), self:ColourTupleToTurbineColour(data["Font_Stack_Colour"]), data["BorderSize"], self:ColourTupleToTurbineColour(data["BorderColour"]), self:ColourTupleToTurbineColour(data["HighlightColour"]), data["HighlightSize"]), value["Time"])
                    end
                    buffbar:ToggleLabel(key, true, value["Time"])
                end
            end))
        end
    end

    if tablelength(data["EnemyHits"]) > 0 then
        for key, value in pairs(data["EnemyHits"]) do
            table.insert(self.enemyChatCallback, AddCallback(ChatEventHandler, "EnemyCombat", function(args)
                local value = data["EnemyHits"][args.SkillName]
                if value then  
                    local highlight = data["AllHighlight"] or value["Highlight"]
                    if not(buffbar:LabelExists(key)) then
                        buffbar:AddLabel(key, ClassSpecificTools.ClassTimerLabel(buffbar, 0, 0, 32, 32, value["Icon"], false, false, nil, 1, 1, false, false, true, data["Scale"], key, true, self.alignments[data["TextAlignment"]], self.alignments[data["StackTextAlignment"]], highlight, self:FontToFont(data["Font_Timer"]), self:FontToFont(data["Font_Stack"]), self:ColourTupleToTurbineColour(data["Font_Timer_Colour"]), self:ColourTupleToTurbineColour(data["Font_Stack_Colour"]), data["BorderSize"], self:ColourTupleToTurbineColour(data["BorderColour"]), self:ColourTupleToTurbineColour(data["HighlightColour"]), data["HighlightSize"]), value["Time"])
                    end
                    buffbar:ToggleLabel(key, true, value["Time"])
                end
            end))
        end
    end

    self.currentHeight = self.currentHeight + (data["BorderSize"] and data["BorderSize"] * 2 or 0) + 32 * data["Scale"]
    table.insert(self.tools, buffbar)

end