function PrimeUI:Conditions(data, parent)
    local valuebar = Turbine.UI.Window()
    valuebar:SetParent(parent)

    valuebar.statusTable = {}
    valuebar.applicationTable = {}
    valuebar.visibilityValue = nil
    valuebar.locationValue = data["Location"] and data["Location"][LocationHandler:GetCurrentLocation()] ~= nil and 1 or 0

    valuebar.togglefunction = function()
        local tableValue = 0
        for key, value in pairs(valuebar.statusTable) do
            if value > 0 then
                if valuebar.applicationTable[key] then
                    if value >= valuebar.applicationTable[key] then
                        tableValue = tableValue + 1
                    end
                else
                    tableValue = tableValue + 1
                end
                
            end
        end
        tableValue = tableValue + valuebar.locationValue

        local visibilityValue = tableValue >= valuebar.minCount
        if valuebar.visibilityValue ~= visibilityValue then
            if visibilityValue then
                for keyword in string.gmatch(data.Activation_Triggers, "([^,]+)") do
                    self:FireTrigger(keyword)
                end
            else
                for keyword in string.gmatch(data.Deactivation_Triggers, "([^,]+)") do
                    self:FireTrigger(keyword)
                end
            end
        end
        valuebar.visibilityValue = visibilityValue
    end

    valuebar.count = 0
    if data["Conjuction"] then
        valuebar.minCount = tablelength(data["Effects"]) + tablelength(data["HealthThreshold"])
        + tablelength(data["Fight"])
        + (tablelength(data["FightPhase"]) > 0 and 1 or 0)
        + (tablelength(data["Location"]) > 0 and 1 or 0)
        + (tablelength(data["Resource"]) > 0 and 1 or 0)
    else
        valuebar.minCount = 1
    end

    if tablelength(data["Effects"]) > 0 then
        for key, value in pairs(data["Effects"]) do
            valuebar.applicationTable[string.format("Effects%s", key)] = value.Applications
            if value.Not_Applied == true then
                valuebar.statusTable[string.format("Effects%s", key)] = 1
            else
                valuebar.statusTable[string.format("Effects%s", key)] = 0
            end
        end

        for i = 1, self.player:GetEffects():GetCount(), 1 do
            local effect = self.player:GetEffects():Get(i)
            local value = data["Effects"][effect:GetName()]
            if value then
                if value.Not_Applied == true then
                    valuebar.statusTable[string.format("Effects%s", effect:GetName())] = valuebar.statusTable[string.format("Effects%s", effect:GetName())] - 1
                else
                    valuebar.statusTable[string.format("Effects%s", effect:GetName())] = valuebar.statusTable[string.format("Effects%s", effect:GetName())] + 1
                end
            end
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local name = effect:GetName()
            if data["Effects"][name] ~= nil then
                local value = data["Effects"][name]
                if value.Not_Applied == true then
                    valuebar.statusTable[string.format("Effects%s", name)] = valuebar.statusTable[string.format("Effects%s", name)] - 1
                else
                    valuebar.statusTable[string.format("Effects%s", name)] = valuebar.statusTable[string.format("Effects%s", name)] + 1
                end
                valuebar.togglefunction()
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local name = effect:GetName()
            if data["Effects"][name] ~= nil then
                local value = data["Effects"][effect:GetName()]
                if value.Not_Applied == true then
                    valuebar.statusTable[string.format("Effects%s", name)] = valuebar.statusTable[string.format("Effects%s", name)] + 1
                else
                    valuebar.statusTable[string.format("Effects%s", name)] = valuebar.statusTable[string.format("Effects%s", name)] - 1
                end

                valuebar.togglefunction()
            end
        end))
    end

    if tablelength(data["FightPhase"]) > 0 then
        table.insert(self.fightEventEndCallbacks, AddCallback(InstanceController, "FightEventEnd", function(args)
            for key, value in pairs(data["FightPhase"]) do
                if value.Boss_Name == args.Boss_Name and (value.Tier_Info == true or tostring(value.Tier_Info) == args.Boss_Tier) then
                    valuebar.statusTable[string.format("FightPhase%s%s", value.Boss_Name, tostring(value.Tier_Info))] = 0
                end
            end
        end))
    end

    if tablelength(data["FightPhase"]) > 0 then
        for key, value in pairs(data["FightPhase"]) do
            if value.Not_Applied == true then
                valuebar.statusTable[string.format("FightPhase%s%s", value.Boss_Name, tostring(value.Tier_Info))] = 1
            else
                valuebar.statusTable[string.format("FightPhase%s%s", value.Boss_Name, tostring(value.Tier_Info))] = 0
            end
        end
        table.insert(self.fightEventPhaseCallback, AddCallback(InstanceController, "FightEventPhase", function(args)
            ChatOutput(PrintTable(args))
            for key, value in pairs(data["FightPhase"]) do
                if value.Boss_Name == args.Boss_Name and (value.Tier_Info == true or tostring(value.Tier_Info) == args.Boss_Tier) then
                    if value.Phase == args.Phase then
                        valuebar.statusTable[string.format("FightPhase%s%s", value.Boss_Name, tostring(value.Tier_Info))] = 1
                    else
                        valuebar.statusTable[string.format("FightPhase%s%s", value.Boss_Name, tostring(value.Tier_Info))] = 0
                    end
                end
            end
        end))
    end

    if tablelength(data["HealthThreshold"]) > 0 then
        local health = self.player:GetMorale() / self.player:GetMaxMorale() * 100
        valuebar.healthThresholdPercentage = {}
        for key, value in pairs(data["HealthThreshold"]) do
            local checkValue = (self.compareFunctions[value["Comparator"]](health, value.Number) and 1) or 0
            valuebar.statusTable[tostring(key) .. tostring(value["Comparator"])] = checkValue
            valuebar.healthThresholdPercentage[tostring(key) .. tostring(value["Comparator"])] = checkValue
        end
        table.insert(self.playerHealthCallbacks, AddCallback(self.player, "MoraleChanged", function()
            local percentage = self.player:GetMorale() / self.player:GetMaxMorale() * 100
            for key, value in pairs(data["HealthThreshold"]) do
                local id = tostring(key) .. tostring(value["Comparator"])
                if self.compareFunctions[value["Comparator"]](percentage, value.Number) then
                    if valuebar.healthThresholdPercentage[id] == 0 then
                        if valuebar.healthThresholdPercentage[id] == 0 then
                            valuebar.healthThresholdPercentage[id] = 1
                            valuebar.statusTable[tostring(key) .. tostring(value["Comparator"])] = valuebar.statusTable[tostring(key) .. tostring(value["Comparator"])] + 1
                        end
                    end
                else
                    if valuebar.healthThresholdPercentage[id] == 1 then
                        valuebar.healthThresholdPercentage[id] = 0
                        valuebar.statusTable[tostring(key) .. tostring(value["Comparator"])] = valuebar.statusTable[tostring(key) .. tostring(value["Comparator"])] - 1
                    end
                end
            end
            valuebar.togglefunction()        
        end))
    end

    if tablelength(data["Resource"]) > 0 then
        for key, value in pairs(data["Resource"]) do
            valuebar.statusTable[value.Comparator .. value.Number] = 0
        end

        valuebar.resourceFunction = function(resourceNumber)
            for key, value in pairs(data["Resource"]) do
                valuebar.statusTable[value.Comparator .. value.Number] = self.compareFunctions[value.Comparator](resourceNumber, value.Number) and 1 or 0
            end
            valuebar.togglefunction()  
        end

        if self.playerClass == Turbine.Gameplay.Class.Beorning then
            valuebar.resourceFunction(self.ClassAttr:GetWrath())
            if not(self.attrCallbacks["WrathChanged"] ) then self.attrCallbacks["WrathChanged"] = {} end
            table.insert(self.attrCallbacks["WrathChanged"], AddCallback(self.ClassAttr, "WrathChanged", function(sender, args) valuebar.resourceFunction(self.ClassAttr:GetWrath()) end))
        elseif self.playerClass == Turbine.Gameplay.Class.Champion then
            valuebar.resourceFunction(self.ClassAttr:GetFervor())
            if not(self.attrCallbacks["FervorChanged"] ) then self.attrCallbacks["FervorChanged"] = {} end
            table.insert(self.attrCallbacks["FervorChanged"], AddCallback(self.ClassAttr, "FervorChanged", function(sender, args) valuebar.resourceFunction( self.ClassAttr:GetFervor() ) end))
        elseif self.playerClass == Turbine.Gameplay.Class.Hunter then
            valuebar.resourceFunction(self.ClassAttr:GetFocus())
            if not(self.attrCallbacks["FocusChanged"] ) then self.attrCallbacks["FocusChanged"] = {} end
            table.insert(self.attrCallbacks["FocusChanged"], AddCallback(self.ClassAttr, "FocusChanged", function(sender, args) valuebar.resourceFunction( self.ClassAttr:GetFocus() ) end))
        elseif self.playerClass == Turbine.Gameplay.Class.RuneKeeper then
            if not(self.attrCallbacks["AttunementChanged"] ) then self.attrCallbacks["AttunementChanged"] = {} end
            valuebar.resourceFunction(self.ClassAttr:GetAttunement())
            table.insert(self.attrCallbacks["AttunementChanged"], AddCallback(self.ClassAttr, "AttunementChanged", function(sender, args) valuebar.resourceFunction( self.ClassAttr:GetAttunement() ) end))
        end
    end

    if tablelength(data["Location"]) > 0 then
        table.insert(self.locationCallbacks, AddCallback(LocationHandler, "Location", function(args)
            valuebar.locationValue = data["Location"][args.Location] and 1 or 0
            valuebar.togglefunction()  
        end))
    end

    valuebar.togglefunction()

    table.insert(self.UI, valuebar)
end